"""Base SDK constants.

Common constants used across base SDK components.
"""

ANONYMOUS_HEADER_PRIVATE_KEY = """-----BEGIN ENCRYPTED PRIVATE KEY-----
MIGjMF8GCSqGSIb3DQEFDTBSMDEGCSqGSIb3DQEFDDAkBBApd1A1zS2kYA2PacTL
ga12AgIIADAMBggqhkiG9w0CCQUAMB0GCWCGSAFlAwQBKgQQLqgqVXR/1YxTjbJ6
wleoyARAf2ENSGpjGhuZ1b1j4Y7YvRyzDNC5Nw5FQfVSll9BkyBP5fYv0+zRuUqu
XE1mIsWI/vEuRcDP/U/EG46uOfaDKg==
-----END ENCRYPTED PRIVATE KEY-----"""

ANONYMOUS_HEADER_PRIVATE_KEY_PASSWORD = b"file_password"

ANONYMOUS_HEADER_PUBLIC_KEY = """-----BEGIN PUBLIC KEY-----
MCowBQYDK2VwAyEArzeACVckM68Jf83BVPzX4ZNWFkyAvbsOwadn/lg7l0c=
-----END PUBLIC KEY-----"""
